-- Rule criteria should take a target info table and one additional argument or table of args, and return a boolean. Yes I know I don't have to do all the if/then/else/return true/false crap. I'm doing it so weird bugs dont crop up elsewhere.

condhail.criteria = {}

-- dummy
condhail.criteria.dummy = {}
condhail.criteria.dummy.usage = [[
dummy - Always returns true 
]]
function condhail.criteria.dummy:match(ti, condition)
	return true 
end
function condhail.criteria.dummy:parseargs(args)
end

-- guild
condhail.criteria.guild = {}
condhail.criteria.guild.usage = [[
guild <guildtag> - Returns true if the target's guild tag matches exactly.
]]
function condhail.criteria.guild:match(ti, guildtag)
    if ti.guild == guildtag then
        return true
    else
        return false
    end
end
function condhail.criteria.guild:parseargs(args)
end

-- name
condhail.criteria.name = {}
condhail.criteria.name.usage = [[
name <playername> - Returns true if target's name matches exactly.
]]
function condhail.criteria.name:match(ti, name)
    if ti.name == name then
        return true
    else
        return false
    end
end
function condhail.criteria.name:parseargs(args)
end

-- npc
condhail.criteria.npc = {}
condhail.criteria.npc.usage = [[
npc - returns true if target's name starts with an asterisk (*).
]]
function condhail.criteria.npc:match(ti, _)
    if ti.name:sub(1,1) == "*" then
        return true
    else
        return false
    end
end
function condhail.criteria.npc:parseargs(args)
end

-- faction
condhail.criteria.faction = {}
condhail.criteria.faction.usage = [[
faction <factionid>  - returns true if the target's factionid matches 
Valid numbers for <factionid>:
1 = Itani
2 = Serco 
3 = UIT
]]
function condhail.criteria.faction:match(ti,faction)
    if ti.factionid == faction then
        return true
    else
        return false
    end
end
function condhail.criteria.faction:parseargs(args)
end

-- standing
condhail.criteria.standing = {}
condhail.criteria.standing.usage = [[
standing <factionid> <standing> <comp> - returns true if the target's standing with the
specified faction is less than or greater than the value provided. Uses real standing 
numbers which can be converted to the UI standing with: 
math.floor(((2000*standing)/65535)-1000)

For example, +600 standing (Admired) would be a value of 52428 for <standing>

<comp> is the comparison operator to use which can be any of <, >, <=, and >=
]]
function condhail.criteria.standing:match(ti,args)
    local faction, value, comp = unpack(args)
    if comp == "<" and ti.standing[faction] < value then
        return true
    elseif comp == "<=" and ti.standing[faction] <= value then
        return true
    elseif comp == ">" and ti.standing[faction] > value then
        return true
    elseif comp == ">=" and ti.standing[faction] >= value then
        return true
    else
        return false
    end
end
function condhail.criteria.standing:parseargs(args)
end

-- cargo
condhail.criteria.cargo = {}
condhail.criteria.cargo.usage = [[
cargo <cargoname> - Returns true if you have a cargo scanner and the last scan 
of your target contains the specified cargo.
]]
function condhail.criteria.cargo:match(ti, cargoname)
    local scanner = false
    for port=1, GetActiveShipNumAddonPorts() do
        local addon = GetInventoryItemName(GetActiveShipItemIDAtPort(port))
        if addon == "Cargo Scanner" then
            scanner = true
        end
    end
    if GetActiveShipName() == "Corvus Greyhound" then
        scanner = true
    end
    if scanner then
        if condhail.lastscan.charid == ti.charid 
            and condhail.lastscan.scandata:find(cargoname, 1, true) then
            return true
        end
    else
        return false
    end
end
function condhail.criteria.cargo:parseargs(args)
end

-- buddy
condhail.criteria.buddy = {}
condhail.criteria.buddy.usage = [[
buddy - Returns true if target is on your buddy list
]]
function condhail.criteria.buddy:match(ti, blah)
    if GetBuddyInfo(ti.charid) then
        return true
    else
        return false
    end
end
function condhail.criteria.buddy:parseargs(args)
end

-- ship
condhail.criteria.ship = {}
condhail.criteria.ship.usage = [[
ship <shipname> Returns true if the target's ship type contains the 
specified name. Example: "Behemoth Heavy Miner MkII" matches for <shipname> Behemoth
]]
function condhail.criteria.ship:match(ti, ship)
    if ti.ship:find(ship, 1, true) then
        return true
    else
        return false
    end
end
function condhail.criteria.ship:parseargs(args)
end

-- sector
condhail.criteria.sector = {}
condhail.criteria.sector.usage = [[
sector <sectorid> - returns true if you (and your target) are currently in the specified sector.

This is the numeric sector ID as used by the game internally. There's a map here:
http://ratelis.com/map/uni.html
]]
function condhail.criteria.sector:match(ti, sectorid)
    if GetCurrentSectorid() == sectorid then
        return true
    else 
        return false
    end
end
function condhail.criteria.sector:parseargs(args)
end

