declare("condhail", {})

-- Vars and config.
condhail.version = "0.2.1"
condhail.debug = true
condhail.systemnotes = 26634245

-- Stores the last scan event data for later comparison
condhail.lastscan = {}
condhail.lastscan.charid = nil
condhail.lastscan.scandata = ""

-- Load rule functions and default rules 
condhail.rules = {}
dofile("rules.lua")

-- Load criteria functions
condhail.criteria = {}
dofile("criteria.lua")

local stringify = function(...)
    --return tostring(...) -- quite frankly this sucks ass.
   return spickle({...})  
end

function condhail:dprint(arg)
    if condhail.debug then
        console_print(arg)
    end
end

function condhail:targetinfo()
    -- Get target info, but more than GetTargetInfo()
    local ti = {} --Target Info
    ti.name, ti.health, ti.dist, ti.factionid, ti.guild, ti.ship = GetTargetInfo()
    ti.charid = GetCharacterIDByName(ti.name)
    ti.nodeid = GetPlayerNodeID(ti.charid)
    ti.standing = {}
    for factionid, _ in pairs(FactionName) do -- get standings (only works for 1, 2, and 3)
        ti.standing[factionid] = GetPlayerFactionStanding(factionid, ti.charid)
    end
    ti.standing["sector"] = GetPlayerFactionStanding("sector", ti.charid) -- get local standing
    return ti
end

function condhail:hail()
    local ti = condhail:targetinfo()
    condhail:dprint ("Got target info: "..stringify(ti)) --DEBUG
    if ti.name == nil then
        print("\127ffffffcondhail: You don't have a target!\127o")
        return false
    end
    for ruleid, rule in ipairs(condhail.rules) do
        local rulename, rulehail, criteria = unpack(rule)
        local rulematch = true
        condhail:dprint ("Evaluating rule ID#"..ruleid..": "..rulename) --DEBUG
        for _, m in ipairs(criteria) do
            local crit = m[1]
            local args = m[2]
            condhail:dprint ("Processing criteria for rule ID#"..ruleid..": criteria["..crit.."](self, ti,"..stringify(args)..")") --DEBUG
            rulematch = condhail.criteria[crit]:match(ti, args) -- do the nasty
            condhail:dprint ("Critera returned:"..stringify(rulematch)) --DEBUG
            if not rulematch then
                break
            end
        end
        if rulematch then
            local hail = rulehail:gsub('%%target%%', ti.name)
            print ("\127ffffffRule: \127ff0000"..rulename.."\127ffffff matched. Sending hail: \127ff0000"..hail.."\127o") 
            SendChat(hail,"PRIVATE",ti.name)
            break
        end

    end

end

-- Event Receivers
condhail.evt = {}

function condhail.evt:TARGET_SCANNED(event, data)
    -- Event: TARGET_SCANNED - Data:
    -- 1: (string)"Emergency Repair Kit x60 \n"
    -- 2: (number) 17
    -- 3: (number) 329
    local ti = condhail:targetinfo()
    condhail.lastscan.charid = ti.charid
    condhail.lastscan.scandata = data

end

function condhail.evt:PLAYER_ENTERED_GAME(event, data)
    condhail.loadrules()
end

function condhail.evt:PLUGINS_LOADED(event, data)
    pcall(function() if test and test.stringify then
        stringify = test.stringify
        if condhail.debug then print("CondHail: DevKit detected, using stringify for "..stringify({"d","e","b","u","g"})) end
    end end)
end

RegisterEvent(condhail.evt, "TARGET_SCANNED")
RegisterEvent(condhail.evt, "PLAYER_ENTERED_GAME")
RegisterEvent(condhail.evt, "PLUGINS_LOADED")
	
-- Console commands
local usage = "\127ffffff"..[[
CondHail - Conditional hail. Picks a hail message based on rules and criteria that match your target
Version ]]..condhail.version..[[ by AbortRetryFail.

Usage:
/condhail: hails target
/condhail help: display this usage
/condhail save: save current rules to systemnotes
/condhail load: discard current rules and load from systemnotes or defaults
/condhail rule list: list rules
/condhail rule add <name> "<message>": Add a new rule to the end of the list. Yes, quotes are necessary.
/condhail rule name <n> <name>: Change the name of rule <n> to <name>
/condhail rule message <n> "<message>": Change the hail message sent for rule <n> to <message>
/condhail rule move <n> <to>: Move rule <n> to before <to> in the list
/condhail rule del <n>: Remove rule <n>
/condhail rule listc <n>: list the critera for rule <n>
/condhail rule addc <n> <crit> <args>: add criteria to rule <n>
/condhail rule delc <n> <m> : delete criteria <m> from rule <n>. Use listc to find <m> 
/condhail criteria list: List available criteria functions 
/condhail criteria <crit> help: Display a description and help(if any) for the specified criteria function ]].."\127o"

local actions = {}
actions.help    = function() print(usage) end
actions.save    = function() condhail:saverules() end
actions.load    = function() condhail:loadrules() end
actions.rule    = {}
actions.rule.list    = function(args)
    -- Print out the rule list to the chat box
    print("\127ffffff---CondHail rule list---\127o")
    for id, rule in ipairs(condhail.rules) do
        local rulename, rulehail, criteria = unpack(rule)
        print("\127ffffff[\127ff7700"..id.."\127ffffff] \127999999"..rulename.."\127ffffff : \127999999"..rulehail.."\127o")
    end
end
actions.rule.add     = function(args)
    if #args == 2 then
        local name, message = unpack(args)
        condhail:addrule(name, message)
    else actions.wat(args[1] or "nil")
    end
end
actions.rule.name    = function(args) 
    if #args == 2 then 
        local ruleid, name = unpack(args)
        -- Set name of rule id args[1]
        condhail.rules[tonumber(ruleid)][1] = name
    else actions.wat(args[1] or "nil")
    end
end
actions.rule.message = function(args)
    if #args == 2 then 
        local ruleid, message = unpack(args)
        -- Set message of rule id args[1]
        condhail.rules[tonumber(ruleid)][2] = message
    else actions.wat(args[1] or "nil")
    end
end
actions.rule.move    = function(args) 
    if #args == 2 then
        local from, to = unpack(args)
        condhail:moverule(from, to)
    else actions.wat(args[1] or "nil")
    end
end
actions.rule.del     = function(args) 
    if #args == 1 then 
        local ruleid = args[1]
        condhail:delrule(ruleid)
    else actions.wat(args[1] or "nil")
    end
end
actions.rule.listc   = function(args) 
    local id = tonumber(args[1]) or nil 
    if id and condhail.rules[id] then
        local rulename, rulehail, criteria = unpack(condhail.rules[id])
        print("\127999999---Criteria for rule: \127ffffff[\127ff7700"..id.."\127ffffff] \127999999"..rulename.."---")
        for i, c in ipairs(condhail.rules[id][3]) do
            local crit, args = unpack(c)
            if type(args) == "table" then
                args = table.concat(args," ")
            end
            print("\127ffffff[\12700ff77"..tostring(i).."\127ffffff] \127999999"..crit.." "..args.."\127o")
        end
   else actions.wat(args[1] or "nil")
   end
end
actions.rule.addc    = function(args) 
    if args[1] and #args >= 3 then
        local id = tonumber(table.remove(args or {},1)) or nil
        local crit = tostring(table.remove(args or {},1)):lower() or nil
        if #args == 1 then args = args[1] end
        if condhail.rules[id] and condhail.criteria[crit] then
            table.insert(condhail.rules[id][3],{crit, args})
        end
    else actions.wat(args[1] or "nil")
    end
end
actions.rule.delc    = function(args) 
    if args[1] and #args == 2 then
        local ruleid, critid = unpack(args)
        ruleid = tonumber(ruleid)
        critid = tonumber(critid)
        if condhail.rules[ruleid] and condhail.rules[ruleid][3][critid] then
            table.remove(condhail.rules[ruleid][3], critid)
        end
    else actions.wat(args[1] or "nil")
    end
end
actions.criteria     = function(args) 
    if condhail.criteria[args[1]] then 
        print(condhail.criteria[args[1]].usage)
    else
        print("\127999999---CondHail criteria list---\127o")
        for crit, _ in pairs(condhail.criteria) do
            print(crit)
        end
    end
end
actions.wat     = function(cmd)
    print("Don't know what to do with '"..cmd.."'. Try /condhail help.")
end

function condhail.act(args, act)
    condhail:dprint("Got args: "..stringify(args))

    -- Here we actually do something 
    if not args[1] then
        condhail:hail()
    else 
        local cmd = tostring(table.remove(args,1)):lower()
        if act[cmd] and type(act[cmd]) == "function" then 
            act[cmd](args)
        elseif act[cmd] and type(act[cmd]) == "table" then
            act=act[cmd] 
            condhail.act(args,act) -- Recurse!
        else
            actions.wat(cmd)
        end
    end

end

function condhail.cmd(_, args)
    args = args or {} 
    condhail.act(args,actions)
end

RegisterUserCommand('condhail', condhail.cmd)

